import Foundation

import Model
import CustomTypes

var rules = FourInARowRules()

var board = rules.createBoard()

guard rules.isValid(board: board) else {
    fatalError("Board is invalid")
}

print(rules.state)

print("All moves:")
for move in rules.validMoves(board: board) {
    print(move)
}

print("Moves for \(Player.A):")
for move in rules.validMoves(board: board, for_player: .A) {
    print(move)
}

print(board)

while case .Playing(let turn) = rules.state {
    let moves = rules.validMoves(board: board, for_player: turn)
    let action = moves[Int.random(in: 0..<moves.count)]
    guard case .InsertOnSide(let side, let offset) = action else {
        fatalError("Unexpected move!")
    }
    
    let move = Move(player: turn, action: action)
    guard rules.isValid(board: board, move: move) else {
        fatalError("Move is in fact invalid???")
    }
    
    switch board.fallCoordinates(
        initialCoords: board.getInsertionCoordinates(from: side, offset: offset),
        direction: .Bottom
    ) {
    case .Border(let at), .Piece(let at, _):
        board[at] = Piece(owner: turn)
    case .Occupied:
        fatalError("Occupied???")
    }
    
    rules.onMoveDone(move: move, board: board)
    
    print(board)
}

if case .Finished(let winner) = rules.state {
    if let winner = winner {
        print("\(winner) is the winner")
    } else {
        print("Draw.")
    }
} else {
    fatalError()
}

print("Moves were:")
for move in rules.history {
    print("\(move.player) did \(move.action)")
}

exit(EXIT_SUCCESS)
